@@ -27,10 +27,6 @@
 #include <limits.h>
 #include <sys/stat.h>
 
-#if defined(__CYGWIN__)
-#include <sys/stat.h>
-#endif
-
 #ifdef __LINUX__
 
 /*
@@ -168,8 +164,6 @@ int main(int argc, char* argv[])
 
     /// オプション処理 ///
     char send_cmd[BUFSIZ];
-    char file_name[PATH_MAX];
-    file_name[0] = 0;
 
     setenv("LINEFIELD","LF", 1);
     enum eTerminalKanjiCode code = kTKUtf8;
@@ -178,12 +172,14 @@ int main(int argc, char* argv[])
     mygetcwd(cwd);
     setenv("PWD", cwd, 1);
 
+    vector_obj* argv2 = VECTOR_NEW(10);
+
     int i;
     enum eRuntimeScript runtime_script = kRSObject;
     for(i=1 ; i<argc; i++){
         // ファイル名
         if(argv[i][0] != '-') {
-            strcpy(file_name, argv[i]);
+            vector_add(argv2, STRING_NEW(argv[i]));
         }
         // 引数
         else {
@@ -259,7 +255,9 @@ int main(int argc, char* argv[])
     }
 
     /// 引数のファイル名がディレクトリじゃなければスクリプトと判断する ///
-    if(strcmp(file_name, "") != 0) {
+    if(vector_size(argv2) > 0) {
+        char* file_name = string_c_str(vector_item(argv2, 0));
+
         struct stat stat_;
         if(stat(file_name, &stat_) < 0) {
             printf("argument file name is invalid\n");
@@ -270,20 +268,39 @@ int main(int argc, char* argv[])
             mcurses_init(code);
             saphire_set_signal();
             saphire_init(kATConsoleApp, FALSE, runtime_script);
+
+            saphire_set_global_var("SCRIPT_FILE_NAME", file_name);
+
+            if(vector_size(argv2) >= 2) {
+                vector_obj* argv3 = VECTOR_NEW(50);
+                int j;
+                for(j=1; j<vector_size(argv2); j++) {
+                    vector_add(argv3, STRING_NEW(string_c_str(vector_item(argv2, j))));
+                }
+                saphire_set_array("ARGV", argv3);
+            }
+            else {
+                saphire_set_array("ARGV", VECTOR_NEW(10));
+            }
+
             sWFd* pipeout = sWFd_new(STDOUT_FILENO);
             sRFd* pipein = sRFd_new(STDIN_FILENO);
             int rcode = saphire_load(file_name, pipeout, pipein, STDERR_FILENO);
             sWFd_flash(pipeout);
             sWFd_delete(pipeout);
             sRFd_delete(pipein);
-            if(rcode != 0) {
+            if(rcode == -1) {
                 fprintf(stderr, "run time err\n");
                 fprintf(stderr, "%s", string_c_str(gErrMsg));
             }
             saphire_final();
             mcurses_final();
-            CHECKML_END();
 
+            for(i=0; i<vector_size(argv2); i++) {
+                string_delete(vector_item(argv2, i));
+            }
+            vector_delete(argv2);
+            CHECKML_END();
             return rcode;
         }
     }
@@ -299,19 +316,27 @@ int main(int argc, char* argv[])
         sWFd_flash(pipeout);
         sWFd_delete(pipeout);
         sRFd_delete(pipein);
-        if(rcode < 0) { 
+        if(rcode == -1) { 
             fprintf(stderr, "run time err\n");
             fprintf(stderr, "%s", string_c_str(gErrMsg));
         }
         saphire_final();
         mcurses_final();
-        CHECKML_END();
 
+        for(i=0; i<vector_size(argv2); i++) {
+            string_delete(vector_item(argv2, i));
+        }
+        vector_delete(argv2);
+        CHECKML_END();
         return rcode;
     }
 
 #if !defined(ENABLE_READLINE)
     fprintf(stderr, "if you use saphire as interactive shell, use sash command instead\n");
+    for(i=0; i<vector_size(argv2); i++) {
+        string_delete(vector_item(argv2, i));
+    }
+    vector_delete(argv2);
     return 0;
 #else
     /// インタラクティブシェル開始 ///
@@ -379,7 +404,7 @@ int main(int argc, char* argv[])
             sWFd_flash(pipeout);
             sWFd_delete(pipeout);
             sRFd_delete(pipein);
-            if(rcode < 0) {
+            if(rcode == -1) {
                 /// ヒストリに追加 ///
                 add_history(buf);
                 history_num++;
@@ -424,10 +449,14 @@ int main(int argc, char* argv[])
     
     mreset_tty();
 
+    for(i=0; i<vector_size(argv2); i++) {
+        string_delete(vector_item(argv2, i));
+    }
+    vector_delete(argv2);
+
     /// メモリリーク検出 ///
     CHECKML_END();
 #endif
-
     return 0;
 }
 
