@@ -40,10 +40,21 @@ enum eCommandKind {
     kDef,
     kPTee,
     kPCat,
+    kReturn,
+    kMInitScr,
+    kMEndWin,
+    kMClear,
+    kMClearImmediately,
+    kMRefresh,
+    kMMove ,
+    kMMoveImmediately,
+    kMPrintw, 
+    kSweep,
+    kShow,
     kCommand 
 };
 
-void err_msg(char* msg);
+void err_msg(char* msg, char* sname, int line);
     // 顼å
 
 string_obj* gAncestalMemoryPipeIn;
@@ -58,16 +69,6 @@ extern char* gStatmentKindStrs[kCommand+1];
 extern hash_obj* gFuncs;
     // 桼ؿ sFunction*äƤ
     // ϴؿ̾
-extern hash_obj* gGlobals;
-    // Хѿ ѿ̾ Ƥstring_obj*
-extern hash_obj* gArrays;
-    // Хѿ ѿ̾
-    // Ƥ(vector_obj*) ˤstring_obj*
-    // äƤ
-hash_obj* gHashs;
-    // Хѿ ѿ̾
-    // Ƥϥϥå(hash_obj*) ϥåˤstring_obj*
-    // äƤ
 
 extern vector_obj* gStackFrame;
     // ѿΥå
@@ -78,10 +79,6 @@ extern enum eAppType gAppType;
     // ץꥱμ
 extern BOOL gJobControl;
     // ֤Υȥ򤹤뤫ɤ
-extern string_obj* gFName;
-    // ץȤΥե̾
-extern int gLine;
-    // ץȤιֹ
 extern vector_obj* gJobs;
     //  sJob*
 extern list_obj* gSigChldRecieved;  
@@ -95,12 +92,13 @@ extern hash_obj* gFileHandles;
     //  եǥץ
     // ֥ȥեΥޥåʥС
 
-extern int gGPipeStat; // 0:̵ 1: ꡼
-    // Хѥפξ
+extern hash_obj* gReadBuffers;
+    //  ե̾
+    //  sRFd*
+    // readѤΥեΥ꡼ɥХåե
 
-string_obj* gGlobalPipe;
-    // Хѥפɤ߹ߤǡ
-    // Хѥ
+vector_obj* gGlobalPipes;
+    // å֥륰Хѥפɤ߹ߤǡ
 
 extern hash_obj* gRegexs;
     // ɽΥå
@@ -366,8 +364,8 @@ typedef struct
 
     BOOL mBackground;        // Хå饦ɤǼ¹
 
-    BOOL mGlobalPipeIn;   // ХѥפϤ뤫
-    BOOL mGlobalPipeOut;  // ХѥפνϤ뤫
+    int mGlobalPipeIn;   // ХѥפϤ뤫 1:  2: å֥
+    int mGlobalPipeOut;  // ХѥפνϤ뤫 1:  2: å֥
 } sStatment;
 
 #ifdef MDEBUG
@@ -452,16 +450,16 @@ void sigchld_block(int block);
 ////////////////////////////////////////////////////////////
 /// from saphire_commands.c
 ////////////////////////////////////////////////////////////
-BOOL statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, sWFd* nextout, sRFd* nextin, int nexterr, int j, vector_obj* cprogs, char* title, BOOL input);
-BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers);
+int statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, sWFd* nextout, sRFd* nextin, int nexterr, int j, vector_obj* cprogs, char* title, BOOL input, BOOL* return_, char* sname, int line);
+BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers,char* sname, int line);
 
 ////////////////////////////////////////////////////////////
 /// from saphire_parser.c
 ////////////////////////////////////////////////////////////
-BOOL expand_env(char* p, string_obj* cmdline2);
-int parse(char* cmdline, char* fname, sStatments* statments);
-BOOL read_expand_brace_expansion(char* p, string_obj* cmdline2, char* separator);
-BOOL parse_number(char* p, vector_obj* numbers);
+BOOL expand_env(char* p, string_obj* out, char* sname, int* line);
+BOOL parse(char* cmdline, char* sname, int* line, sStatments* statments);
+BOOL read_expand_brace_expansion(char* p, string_obj* cmdline2, char* separator, char* sname, int* line);
+BOOL parse_number(char* p, vector_obj* numbers, char* sname, int line);
 
 typedef struct {
     int number;
@@ -472,7 +470,7 @@ typedef struct {
 sIndex* sIndex_new(int num, int num2, int num_num);
 void sIndex_delete(sIndex* self);
 
-BOOL read_until_next_cbrace_end(char** p, string_obj* not_evaled, int* read_end_of_statment);
+BOOL read_until_next_cbrace_end(char** p, string_obj* not_evaled, int* read_end_of_statment, BOOL line_inc, char* sname, int* line);
 
 ////////////////////////////////////////////////////////////
 // from saphire_vm.c
@@ -481,4 +479,5 @@ BOOL forground_job(int num);
 void background_job(int num);
 void tmpfile_all_sweep();
 
+
 #endif
