@@ -75,6 +75,17 @@ char* gStatmentKindStrs[kCommand+1] = {
     "def",
     "ptee",
     "pcat",
+    "return",
+    "minitscr",
+    "mendwin",
+    "mclear",
+    "mclear_immediately",
+    "mrefresh",
+    "mmove",
+    "mmove_immediately",
+    "mprintw",
+    "sweep",
+    "show",
     "command"
 };
 
@@ -101,10 +112,6 @@ enum eAppType gAppType;
     // アプリケーションの種類
 BOOL gJobControl;
     // ジョブのコントロールをするかどうか
-string_obj* gFName;
-    // スクリプトのファイル名
-int gLine;
-    // スクリプトの行番号
 vector_obj* gJobs;
     // ジョブ sJob*の配列
 list_obj* gSigChldRecieved;  
@@ -117,8 +124,13 @@ hash_obj* gFileHandles;
     // キー ファイル名
     // 内容 ファイルディスクリプタ
 
-string_obj* gGlobalPipe;
-    // グローバルパイプのデータ
+hash_obj* gReadBuffers;
+    // キー ファイル名
+    // 内容 sRFd*
+    // read用のファイルのリードバッファー
+
+vector_obj* gGlobalPipes;
+    // スタッカブルグローバルパイプのデータ
 
 hash_obj* gRegexs;
     // 正規表現のキャッシュ
@@ -297,6 +309,15 @@ void update_ary_env(char* key)
         if(str) string_delete(str);
         hash_put(gGlobals, name, STRING_NEW(number));
     }
+    else {
+        char name[128];
+        snprintf(name, 128, "%s_size", key);
+        string_obj* str = hash_item(gGlobals, name);
+        if(str) {
+            string_delete(str);
+            hash_erase(gGlobals, name);
+        }
+    }
 }
 
 // 配列変数にまつわる変数を消す
@@ -312,10 +333,10 @@ void clear_ary_env(char* key)
 }
 
 // エラーメッセージを設定
-void err_msg(char* msg)
+void err_msg(char* msg, char* sname, int line)
 {
-    char* tmp = MALLOC(string_length(gFName) + 128 + strlen(msg));
-    snprintf(tmp, string_length(gFName) + 128 + strlen(msg), "%s %d: %s\n", string_c_str(gFName) , gLine, msg);
+    char* tmp = MALLOC(strlen(sname) + 128 + strlen(msg));
+    snprintf(tmp, strlen(sname) + 128 + strlen(msg), "%s %d: %s\n", sname,  line, msg);
 
     string_put(gErrMsg, tmp);
 
@@ -484,6 +505,7 @@ void saphire_delete_same_name_var(char* name)
         }
         vector_delete(v);
         hash_erase(gArrays, name);
+        update_ary_env(name);
     }
 
     hash_obj* h = hash_item(gHashs, name);
@@ -506,14 +528,8 @@ void saphire_delete_same_name_var(char* name)
 
 void saphire_set_global_var(char* name, char* value)
 {
-    string_obj* global = hash_item(gGlobals, name);
-
-    if(global) {
-        string_put(global, value);
-    }
-    else {
-        hash_put(gGlobals, name, STRING_NEW(value));
-    }
+    saphire_delete_same_name_var(name);
+    hash_put(gGlobals, name, STRING_NEW(value));
 }
 
 char* saphire_get_global_var(char* name)
@@ -538,6 +554,13 @@ vector_obj* saphire_get_array(char* name)
     return hash_item(gArrays, name);
 }
 
+void saphire_set_array(char* name, vector_obj* array)
+{
+    saphire_delete_same_name_var(name);
+    hash_put(gArrays, name, array);
+    update_ary_env(name);
+}
+
 char* saphire_get_local_var(char* name)
 {
     if(vector_size(gStackFrame) == 0) {
@@ -937,11 +960,120 @@ void sVar_delete(sVar* self)
 sVar* sVar_load(int fd)
 {
     sVar* self = MALLOC(sizeof(sVar));
+    int n;
+    char* buf;
+
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    
+    buf = MALLOC(n+1);
+    if(read(fd, buf, n) < 0) {
+        perror("read");
+        exit(1);
+    }
+    buf[n] = 0;
+    self->mName = STRING_NEW(buf);
+    FREE(buf);
+
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    
+    buf = MALLOC(n+1);
+    if(read(fd, buf, n) < 0) {
+        perror("read");
+        exit(1);
+    }
+    buf[n] = 0;
+    self->mIndex = STRING_NEW(buf);
+    FREE(buf);
+
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    
+    buf = MALLOC(n+1);
+    if(read(fd, buf, n) < 0) {
+        perror("read");
+        exit(1);
+    }
+    buf[n] = 0;
+    self->mIndex2 = STRING_NEW(buf);
+    FREE(buf);
+
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    
+    buf = MALLOC(n+1);
+    if(read(fd, buf, n) < 0) {
+        perror("read");
+        exit(1);
+    }
+    buf[n] = 0;
+    self->mDelimiter = STRING_NEW(buf);
+    FREE(buf);
+
     return self;
 }
 
 void sVar_save(sVar* self, int fd)
 {
+    string_obj* str = self->mName;
+    int n = string_size(str);
+
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    if(write(fd, string_c_str(str), n) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    str = self->mIndex;
+    n = string_size(str);
+
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
+    if(write(fd, string_c_str(str), n) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    str = self->mIndex2;
+    n = string_size(str);
+
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    if(write(fd, string_c_str(str), n) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    str = self->mDelimiter;
+    n = string_size(str);
+
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    if(write(fd, string_c_str(str), n) < 0) {
+        perror("write");
+        exit(1);
+    }
 }
 
 sArg* sArg_new(int kind, void* body)
@@ -1007,9 +1139,19 @@ void sArg_save(sArg* self, int fd)
             break;
 
         case 1:
+            sVar_save(self->mBody, fd);
             break;
 
         case 2:
+            sStatments_save(self->mBody, fd);
+            break;
+
+        case 3:
+            sVar_save(self->mBody, fd);
+            break;
+
+        case 4:
+            sStatments_save(self->mBody, fd);
             break;
     }
 }
@@ -1045,9 +1187,19 @@ sArg* sArg_load(int fd)
         break;
 
     case 1:
+        self->mBody = sVar_load(fd);
         break;
 
     case 2:
+        self->mBody = sStatments_load(fd);
+        break;
+
+    case 3:
+        self->mBody = sVar_load(fd);
+        break;
+
+    case 4:
+        self->mBody = sStatments_load(fd);
         break;
     }
 
@@ -1849,6 +2001,7 @@ sStatments* sStatments_new()
 
 void sStatments_view(sStatments* self)
 {
+
     int i;
     for(i=0; i<vector_size(self->mStatments); i++)
         sStatment_view(vector_item(self->mStatments, i));
@@ -2186,6 +2339,7 @@ static void saphire_rehash_without_user_fun()
     (void)vector_sort(gPrograms, name_sort);
 
     hash_delete(check);
+
 }
 
 static void saphire_rehash_user_fun()
@@ -2307,7 +2461,7 @@ static void read_rc_obj_file()
 ////////////////////////////////////////////////////////
 void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript runtime_script)
 {
-    gGlobalPipe = STRING_NEW("");
+    gGlobalPipes = VECTOR_NEW(10);
 
     gAncestalMemoryPipeIn = STRING_NEW("");
     gAncestalMemoryPipeOut = STRING_NEW("");
@@ -2323,12 +2477,10 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gGlobals = HASH_NEW(100);
 
-    gFName = STRING_NEW("");
-    gLine = 0;
-
     gFileHandles = HASH_NEW(100);
+    gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "0.9.2", 1);
+    setenv("SAPHIRE_VERSION", "0.9.5", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2403,9 +2555,13 @@ void saphire_final()
 {
     hash_delete(gMatchRestart);
 
-    string_delete(gGlobalPipe);
-
     int i;
+    int len = vector_size(gGlobalPipes);
+    for(i=0; i<len; i++) {
+        string_delete(vector_item(gGlobalPipes, i));
+    }
+    vector_delete(gGlobalPipes);
+
     hash_it* it = hash_loop_begin(gFuncs);
     while(it != NULL) {
         sFunction_delete(hash_loop_item(it));
@@ -2475,6 +2631,15 @@ void saphire_final()
         it = hash_loop_next(it);
     }
     hash_delete(gFileHandles);
+
+    it = hash_loop_begin(gReadBuffers);
+    while(it) {
+        sRFd* fd = (sRFd*)hash_loop_item(it);
+        sRFd_close(fd);
+        sRFd_delete(fd);
+        it = hash_loop_next(it);
+    }
+    hash_delete(gReadBuffers);
     
     for(i=0; i<vector_size(gPrograms); i++) {
         string_delete(vector_item(gPrograms, i));
@@ -2545,7 +2710,6 @@ void saphire_final()
     vector_delete(gDirStack);
 
     string_delete(gErrMsg);
-    string_delete(gFName);
 
     /// プロセス置換で使った一時ファイルを掃除 ///
     int k;
